#include <stdio.h>
#include <stdlib.h>
#include <SDL/SDL.h>

#define IMG_4BIT 0
#define IMG_8BIT 1
#define IMG_24BIT 2

int bench(int profundidadBmp, unsigned int bucles, SDL_Surface *pantalla, FILE *fichero)
{
    int i;
    Uint32 inicio, fin;
    
    SDL_Surface *bmp, *imagen;
    SDL_Rect dstrect;
    
    // Inicializar el contador:
    inicio = SDL_GetTicks();
    
     // Ejecutar el nmero de bucles indicados por la variable bucles:
    for(i = 0; i < bucles; i++)
    {
        // Cargar el BMP:
        if(profundidadBmp == IMG_4BIT)
            bmp = SDL_LoadBMP("imagen4.bmp");
        else if(profundidadBmp == IMG_8BIT)
            bmp = SDL_LoadBMP("imagen8.bmp");
        else if(profundidadBmp == IMG_24BIT)
            bmp = SDL_LoadBMP("imagen24.bmp");
        else
        {
           fprintf(stderr, "Profundidad de BMP desconocida.\n");
           return 1;
        }
        
        if (bmp == NULL) 
        {
           fprintf(stderr, "Problema: %s\n", SDL_GetError());
           return 1;
        }
    
        // Convertir la imagen al formato de la pantalla:
        imagen = SDL_DisplayFormat(bmp);
        SDL_FreeSurface(bmp);
        if(imagen == NULL) 
        {
           fprintf(stderr, "Problema: %s\n", SDL_GetError());
           return 1;
        }
    
        // Pegar la imagen en pantalla:
        dstrect.x = i % 640;
        dstrect.y = i % 480;
        dstrect.w = imagen->w;
        dstrect.h = imagen->h;
        if(SDL_BlitSurface(imagen, NULL, pantalla, &dstrect) < 0) 
        {
           SDL_FreeSurface(imagen);
           fprintf(stderr, "Problema: %s\n", SDL_GetError());
           return 1;
        }
        SDL_FreeSurface(imagen);
    
        // Actualizar la pantalla:
        SDL_UpdateRects(pantalla, 1, &dstrect);
    }
    
    // Finalizar el contador:
    fin = SDL_GetTicks();
    
    // Imprimir el resultado en el fichero resultado.txt:
    fprintf(fichero, "Tiempo usado: %d\n", fin - inicio);
    
    return 0;
}

int main(int argc, char *argv[])
{
    int error;
    SDL_Surface *pantalla;
    FILE *fichero;

    // Inicializar SDL:
    if(SDL_Init(SDL_INIT_VIDEO) < 0) 
    {
       fprintf(stderr, "Problema: %s\n", SDL_GetError());
       return 1;
    }

    // Inicializar el modo de video:
    pantalla = SDL_SetVideoMode(640, 480, 24, SDL_SWSURFACE);
    if(pantalla == NULL) 
    {
       fprintf(stderr, "Problema: %s\n", SDL_GetError());
       return 1;
    }
    
    // Crear el fichero resultado.txt:
    fichero = fopen("resultado.txt", "w");
    
    // Ejecutar el bench:
    fprintf(fichero, "Imagen de 16 colores      ");
    error = bench(IMG_4BIT, 5000, pantalla, fichero);
    if(error)
        return 1;
    
    fprintf(fichero, "Imagen de 256 colores     ");
    error = bench(IMG_8BIT, 5000, pantalla, fichero);
    if(error)
        return 1;
        
    fprintf(fichero, "Imagen de 24 bits         ");
    error = bench(IMG_24BIT, 5000, pantalla, fichero);
    if(error)
        return 1;
             
    // Cerrar el fichero resultado.txt:   
    fclose(fichero);
    
    return 0;
}
